class RecordException(Exception):

    def __init__(self, **kwargs):
        self.record = kwargs.get('record', {})
        self.message = kwargs.get('message', 'record error')
        super().__init__(self.message)


class EventException(Exception):

    def __init__(self, **kwargs):
        self.message = kwargs.get('message', 'event error')
        super().__init__(self.message)
    

class EventTimeOutException(Exception):

    def __init__(self, **kwargs):
        self.message = kwargs.get('message', 'event timeout')
        super().__init__(self.message)



import json
import unittest

from acai_aws.apigateway.router import Router

from tests.mocks.apigateway import mock_middleware, mock_request


class RouterMappingTest(unittest.TestCase):
    maxDiff = None
    base_path = 'unit-test/v1'
    schema_path = 'tests/mocks/apigateway/openapi.yml'
    base_path_schema_path = 'tests/mocks/apigateway/base_path_openapi.yml'
    basic_event = mock_request.get_basic_post()
    raise_exception_event = mock_request.get_raised_exception_post()
    mock_request = mock_request
    unhandled_exception_event = mock_request.get_unhandled_exception_post()
    expected_open_headers = {
        'Access-Control-Allow-Origin': '*',
        'Access-Control-Allow-Headers': '*'
    }
    handler_mapping = {
        'auto': 'tests/mocks/apigateway/router/mapping_handlers/auto/__init__.py',
        'basic': 'tests/mocks/apigateway/router/mapping_handlers/basic.py',
        'optional-params': 'tests/mocks/apigateway/router/mapping_handlers/optional_params.py',
        'raise-exception': 'tests/mocks/apigateway/router/mapping_handlers/raise_exception.py',
        'unhandled-exception': 'tests/mocks/apigateway/router/mapping_handlers/unhandled_exception.py',
        'nested/reqs': 'tests/mocks/apigateway/router/mapping_handlers/nested/reqs.py',
        'nested/{id}': 'tests/mocks/apigateway/router/mapping_handlers/nested/nested_id.py'
    }

    def test_basic_mapping_routing_works(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path
        )
        result = router.route(self.basic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertFalse(result['isBase64Encoded'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual(self.expected_open_headers, result['headers'])
        self.assertDictEqual({"router_mapping_basic": {"body_key": "body_value"}}, json_dict_response)

    def test_basic_mapping_routing_works_with_verbose_logging(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            verbose_logging=True
        )
        result = router.route(self.basic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertFalse(result['isBase64Encoded'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual(self.expected_open_headers, result['headers'])
        self.assertDictEqual({"router_mapping_basic": {"body_key": "body_value"}}, json_dict_response)

    def test_auto_load_works(self):
        try:
            router = Router(
                base_path=self.base_path,
                handlers=self.handler_mapping,
                schema=self.schema_path
            )
            router.auto_load()
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)

    def test_basic_mapping_routing_works_no_schema_defined(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping
        )
        result = router.route(self.basic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertFalse(result['isBase64Encoded'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual(self.expected_open_headers, result['headers'])
        self.assertDictEqual({"router_mapping_basic": {"body_key": "body_value"}}, json_dict_response)

    def test_basic_mapping_routing_works_with_raised_exception(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            output_error=True
        )
        result = router.route(self.raise_exception_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertFalse(result['isBase64Encoded'])
        self.assertEqual(418, result['statusCode'])
        self.assertDictEqual(self.expected_open_headers, result['headers'])
        self.assertDictEqual({"errors": [{"key_path": "crazy_error", "message": "I am a teapot"}]}, json_dict_response)

    def test_basic_mapping_routing_works_with_unhandled_exception(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            output_error=True
        )
        result = router.route(self.unhandled_exception_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertFalse(result['isBase64Encoded'])
        self.assertEqual(500, result['statusCode'])
        self.assertDictEqual(self.expected_open_headers, result['headers'])
        self.assertDictEqual({'errors': [{'key_path': 'unknown', 'message': "name 'UnknownException' is not defined"}]}, json_dict_response)

    def test_basic_mapping_routing_works_and_before_all_function_called(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            before_all=mock_middleware.mock_before_all,
            schema=self.schema_path
        )
        router.route(self.basic_event, None)
        self.assertTrue(mock_middleware.mock_before_all.has_been_called)

    def test_basic_mapping_routing_works_and_after_all_function_called(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            after_all=mock_middleware.mock_after_all,
            schema=self.schema_path
        )
        router.route(self.basic_event, None)
        self.assertTrue(mock_middleware.mock_after_all.has_been_called)

    def test_basic_mapping_routing_works_and_with_auth_function_called(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            with_auth=mock_middleware.mock_with_auth,
            schema=self.schema_path
        )
        router.route(self.basic_event, None)
        self.assertTrue(mock_middleware.mock_with_auth.has_been_called)

    def test_basic_mapping_routing_works_and_on_error_function_called(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            on_error=mock_middleware.mock_on_error,
            schema=self.schema_path
        )
        router.route(self.raise_exception_event, None)
        self.assertTrue(mock_middleware.mock_on_error.has_been_called)

    def test_basic_mapping_routing_works_and_bad_on_error_function_caught(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            on_error=mock_middleware.mock_on_error_exception,
            schema=self.schema_path
        )
        router.route(self.raise_exception_event, None)
        self.assertTrue(mock_middleware.mock_on_error_exception.has_been_called)

    def test_requirements_decorator_works_and_passes_proper_body_request(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json'},
            path='unit-test/v1/nested/reqs',
            proxy='nested/reqs',
            method='post',
            body={
                'name': 'unit-test',
                'email': 'unit@email.com',
                'phone': 1234567890,
                'active': True
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path
        )
        result = router.route(dynamic_event, None)
        self.assertEqual(200, result['statusCode'])

    def test_requirements_decorator_works_and_fails_improper_body_request(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json'},
            path='unit-test/v1/nested/reqs',
            proxy='nested/reqs',
            method='post',
            body={
                'name': 'unit-test',
                'email': 'unit@email.com',
                'phone': '1234567890',
                'active': True
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path
        )
        result = router.route(dynamic_event, None)
        self.assertEqual(400, result['statusCode'])
        json_dict_response = json.loads(result['body'])
        self.assertDictEqual({'errors': [{'key_path': 'phone', 'message': "'1234567890' is not of type 'number'"}]}, json_dict_response)

    def test_requirements_decorator_works_and_passes_proper_query_request(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json'},
            path='unit-test/v1/nested/reqs',
            proxy='nested/reqs',
            method='get',
            query={
                'auth_id': 'some-id',
                'email': 'some@email.com',
                'name': 'unit-test'
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path
        )
        result = router.route(dynamic_event, None)
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual(self.expected_open_headers, result['headers'])

    def test_requirements_decorator_works_and_fails_improper_query_request_missing_required(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json'},
            path='unit-test/v1/nested/reqs',
            proxy='nested/reqs',
            method='get',
            query={
                'email': 'some@email.com',
                'name': 'unit-test'
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping
        )
        result = router.route(dynamic_event, None)
        self.assertEqual(400, result['statusCode'])
        json_dict_response = json.loads(result['body'])
        self.assertDictEqual({'errors': [{'key_path': 'query_params', 'message': 'Please provide auth_id in query_params'}]}, json_dict_response)

    def test_requirements_decorator_works_and_passes_proper_headers(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json', 'correlation-id': 'abc-123'},
            path='unit-test/v1/nested/reqs',
            proxy='nested/reqs',
            method='delete'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping
        )
        result = router.route(dynamic_event, None)
        self.assertEqual(200, result['statusCode'])

    def test_requirements_decorator_works_and_fails_improper_headers_missing_required(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'correlation-id': 'abc-123'},
            path='unit-test/v1/nested/reqs',
            proxy='nested/reqs',
            method='delete'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(400, result['statusCode'])
        self.assertDictEqual({'errors': [{'key_path': 'headers', 'message': 'Please provide content-type in headers'}]}, json_dict_response)

    def test_requirements_decorator_works_and_fails_improper_headers_unknown_header(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'unknown-id': 'abc-123'},
            path='unit-test/v1/nested/reqs',
            proxy='nested/reqs',
            method='delete'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(400, result['statusCode'])
        self.assertDictEqual(
            {
                'errors': [{'key_path': 'headers', 'message': 'Please provide content-type in headers'},
                           {'key_path': 'headers', 'message': 'unknown-id is not an available headers'}]
            }, json_dict_response
        )

    def test_requirements_decorator_works_and_passes_proper_dynamic_route_request(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json'},
            path='unit-test/v1/nested/abc-123',
            proxy='nested/abc-123',
            method='patch',
            body={
                'test_id': 'unit-test',
                'email': 'unit@email.com'
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual(
            {
                'router_nested_mapping_dynamic': {
                    'test_id': 'unit-test',
                    'email': 'unit@email.com'
                },
                'path_params': {'proxy': 'nested/abc-123', 'nested_id': 'abc-123'}
            },
            json_dict_response
        )

    def test_requirements_decorator_works_and_fails_improper_dynamic_route_request(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json'},
            path='unit-test/v1/nested/abc-123',
            proxy='nested/abc-123',
            method='patch',
            body={
                'email': 'unit@email.com'
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(400, result['statusCode'])
        self.assertDictEqual({'errors': [{'key_path': 'root', 'message': "'test_id' is a required property"}]}, json_dict_response)

    def test_basic_mapping_routing_works_with_validate_response_body(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'x-api-key': 'some-key'},
            path='unit-test/v1/auto',
            proxy='auto',
            method='patch'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual({'page_number': 1, 'data': {'id': '2'}}, json_dict_response)

    def test_auto_validate_works_and_passes_proper_dynamic_route_request(self):
        body = {
            'test_id': 'abc123',
            'object_key': {
                'key': 'value'
            },
            'array_number': [1, 2, 3],
            'array_objects': [
                {
                    'array_string_key': 'string_value',
                    'array_number_key': 0
                }
            ]
        }
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json', 'x-api-key': 'some-key'},
            path='unit-test/v1/auto',
            proxy='auto',
            method='post',
            body=body
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            auto_validate=True
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual({'router_mapping_auto': body}, json_dict_response)

    def test_auto_validate_works_and_fails_improper_dynamic_route_request(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'content-type': 'application/json', 'x-api-key': 'some-key'},
            path='unit-test/v1/auto',
            proxy='auto',
            method='post',
            body={
                'object_key': {
                    'key': 'value'
                },
                'array_number': [1, 2, 3],
                'array_objects': [
                    {
                        'array_string_key': 'string_value',
                        'array_number_key': 0
                    }
                ]
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            auto_validate=True
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(400, result['statusCode'])
        self.assertDictEqual({'errors': [{'key_path': 'root', 'message': "'test_id' is a required property"}]}, json_dict_response)

    def test_auto_validate_works_and_passes_proper_dynamic_route_request_with_optional_params(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'x-api-key': 'some-key'},
            path='unit-test/v1/optional-params',
            proxy='optional-params',
            method='get',
            query={
                'group': 'some-group'
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            auto_validate=True
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual({'router_mapping_optional': True}, json_dict_response)

    def test_auto_validate_works_with_base_path_and_optional_params(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'x-api-key': 'some-key'},
            path='unit-test/v1/optional-params',
            proxy='optional-params',
            method='get',
            query={
                'group': 'some-group'
            }
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.base_path_schema_path,
            auto_validate=True
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual({'router_mapping_optional': True}, json_dict_response)

    def test_basic_mapping_routing_works_with_auto_validate_and_with_auth_function_called(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'x-api-key': 'some-key'},
            path='unit-test/v1/auto',
            proxy='auto',
            method='get'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            with_auth=mock_middleware.mock_with_auth,
            schema=self.schema_path,
            auto_validate=True
        )
        router.route(dynamic_event, None)
        self.assertTrue(mock_middleware.mock_with_auth.has_been_called)

    def test_basic_mapping_routing_works_with_auto_validate_response_body(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'x-api-key': 'some-key'},
            path='unit-test/v1/auto',
            proxy='auto',
            method='get'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            auto_validate=True,
            validate_response=True
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual({'page_number': 1, 'data': {'id': '2'}}, json_dict_response)

    def test_basic_mapping_routing_fails_with_auto_validate_response_body(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'x-api-key': 'some-key'},
            path='unit-test/v1/auto',
            proxy='auto',
            method='put'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            auto_validate=True,
            validate_response=True
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(500, result['statusCode'])
        self.assertDictEqual(
            {
                'errors': [
                    {
                        'key_path': 'root',
                        'message': "'data' is a required property"
                    },
                    {
                        'key_path': 'root',
                        'message': "Additional properties are not allowed ('bad-put' was unexpected)"
                    },
                    {
                        'key_path': 'response',
                        'message': 'There was a problem with the APIs response; does not match defined schema'
                    }
                ]
            },
            json_dict_response
        )

    def test_basic_mapping_routing_fails_with_response_body(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            headers={'x-api-key': 'some-key'},
            path='unit-test/v1/auto',
            proxy='auto',
            method='delete'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_mapping,
            schema=self.schema_path,
            validate_response=True
        )
        result = router.route(dynamic_event, None)
        json_dict_response = json.loads(result['body'])
        self.assertEqual(500, result['statusCode'])
        self.assertDictEqual(
            {
                'errors': [
                    {
                        'key_path': 'root',
                        'message': "'data' is a required property"
                    },
                    {
                        'key_path': 'root',
                        'message': "Additional properties are not allowed ('bad-delete' was unexpected)"
                    },
                    {
                        'key_path': 'response',
                        'message': 'There was a problem with the APIs response; does not match defined schema'
                    }
                ]
            },
            json_dict_response
        )



import base64
import gzip
from io import BytesIO

from acai_aws.common.json_helper import JsonHelper


class Response:

    def __init__(self, **kwargs):
        self.__code = 200
        self.__is_json = True
        self.__cors = kwargs.get('cors', True)
        self.__base64_encoded = False
        self.__compress = False
        self.__content_type = ''
        self.__headers = {}
        self.__body = {}

    @property
    def headers(self):
        if self.cors:
            self.__set_cors()
        return self.__headers

    @headers.setter
    def headers(self, header):
        key, value = header
        self.__headers[key] = value

    @property
    def content_type(self):
        if self.is_json and not self.__content_type and not self.headers.get('content-type', self.headers.get('Content-Type')):
            return 'application/json'
        return self.__content_type if self.__content_type else self.headers.get('content-type', self.headers.get('Content-Type', ''))

    @content_type.setter
    def content_type(self, content_type ):
        self.__content_type = content_type

    @property
    def cors(self):
        return self.__cors

    @cors.setter
    def cors(self, access):
        self.__cors = access
    
    @property
    def open_cors(self):
        return self.__cors

    @open_cors.setter
    def open_cors(self, access):
        self.__cors = access

    @property
    def base64_encoded(self):
        return self.__base64_encoded

    @property
    def compress(self):
        return self.__compress

    @compress.setter
    def compress(self, value):
        self.__compress = value

    @property
    def code(self):
        if isinstance(self.__body, dict) and self.__code == 200 and not self.__body:
            return 204
        if isinstance(self.__body, dict) and self.__code == 200 and self.has_errors:
            return 400
        return self.__code

    @code.setter
    def code(self, code):
        self.__code = code

    @property
    def is_json(self):
        return self.__is_json

    @is_json.setter
    def is_json(self, is_json):
        self.__is_json = is_json

    @property
    def has_errors(self):
        return 'errors' in self.__body

    @property
    def body(self):
        body = JsonHelper.encode(self.__body, raise_error=True) if self.is_json else self.__body
        if self.compress:
            return self.__compress_body(body)
        if isinstance(self.__body, (dict, list, tuple)):
            return body
        return body

    @body.setter
    def body(self, body):
        self.__body = body

    @property
    def raw(self):
        return self.__body

    @property
    def full(self):
        return {
            'body': self.body,
            'headers': self.headers,
            'statusCode': self.code,
            'isBase64Encoded': self.base64_encoded
        }

    def set_error(self, key_path, message):
        error = {'key_path': key_path, 'message': message}
        if isinstance(self.__body, dict) and 'errors' in self.__body:
            self.__body['errors'].append(error)
        else:
            self.__body = {'errors': [error]}

    def __compress_body(self, body):
        self.headers = ('Content-Encoding', 'gzip')
        self.__base64_encoded = True
        bytes_io = BytesIO()
        with gzip.GzipFile(fileobj=bytes_io, mode='w') as file:
            file.write(body.encode('utf-8'))
        return base64.b64encode(bytes_io.getvalue()).decode('ascii')

    def __set_cors(self):
        self.__headers['Access-Control-Allow-Origin'] = '*'
        self.__headers['Access-Control-Allow-Headers'] = '*'

    def __str__(self):
        return str(
            {
                'hasErrors': self.has_errors,
                'response': {
                    'headers': self.full['headers'],
                    'statusCode': self.full['statusCode'],
                    'isBase64Encoded': self.full['isBase64Encoded'],
                    'body': JsonHelper.decode(self.full['body'])
                }
            }
        )



import copy
import json
import inspect

import jsonref
from pydantic import BaseModel
import yaml


class Schema:

    def __init__(self, **kwargs):
        self.__schema = kwargs.get('schema')
        self.__config = kwargs.get('schema_config', {})
        self.__spec = {}

    @property
    def spec(self):
        return self.__spec

    def load_schema_file(self):
        self.__get_full_spec()

    def get_openapi_spec(self):
        return self.__get_full_spec()

    def get_body_spec(self, required_body=None):
        if required_body and inspect.isclass(required_body) and issubclass(required_body, BaseModel):
            return required_body
        elif self.__schema and isinstance(self.__schema, dict):
            body_spec = self.__schema
        elif required_body and isinstance(required_body, dict):
            body_spec = required_body
        elif required_body and isinstance(required_body, str):
            body_spec = self.__get_component_spec(required_body)
        body_spec['additionalProperties'] = self.__config.get('allow_additional_properties', False)
        return body_spec
    
    def get_route_spec(self, route, method):
        return self.__get_route_spec(route, method)        

    def __get_full_spec(self):
        if not self.spec and self.__schema:
            unresolved_spec = self.__get_spec_from_file()
            resolved_spec = jsonref.loads(json.dumps(unresolved_spec), jsonschema=True, merge_props=True)
            self.__spec = self.__combine_all_of_spec(resolved_spec)
        return self.spec

    def __get_spec_from_file(self):
        with open(self.__schema, encoding='utf-8') as schema_file:
            return yaml.load(schema_file, Loader=yaml.FullLoader)    

    def __combine_all_of_spec(self, spec):
        combined = copy.deepcopy(spec)
        return self.__walk_spec(spec, combined)

    def __walk_spec(self, spec, combined_spec):
        for spec_key in spec:
            if spec_key == 'allOf':
                self.__combine_all_of(spec, spec_key, combined_spec)
            elif isinstance(spec[spec_key], dict):
                self.__walk_spec(spec[spec_key], combined_spec[spec_key])
            elif isinstance(spec[spec_key], list):
                self.__iter_spec_list(spec, spec_key, combined_spec)
        return combined_spec

    def __combine_all_of(self, spec, spec_key, combined_spec):
        combined = {
            'type': 'object',
            'properties': {},
            'required': []
        }
        for all_of in spec[spec_key]:
            if isinstance(all_of, dict) and all_of.get('properties'):
                combined['properties'].update(all_of['properties'])
            if isinstance(all_of, dict) and all_of.get('required'):
                combined['required'] += all_of['required']
        if combined['properties']:
            del combined_spec['allOf']
            combined_spec.update(combined)

    def __iter_spec_list(self, spec, spec_key, combined_spec):
        for index, item in enumerate(spec[spec_key]):
            if isinstance(item, dict):
                self.__walk_spec(item, combined_spec[spec_key][index])

    def __get_component_spec(self, required_body=None):
        spec = self.__get_full_spec()
        return spec['components']['schemas'][required_body]

    def __get_route_spec(self, route, method):
        spec = self.__get_full_spec()
        if spec.get('basePath'):
            route = route.replace(spec['basePath'], '')
        return spec['paths'][route][method]



import base64
import json
import unittest

from acai_aws.kinesis.record import Record

from tests.mocks.kinesis import mock_event


class KinesisRecordTest(unittest.TestCase):
    basic_record = mock_event.get_basic()['Records'][0]

    def test_record_accepts_event(self):
        record = Record(self.basic_record)
        self.assertEqual(record.operation, record.UNKNOWN)
        self.assertEqual(record.id, self.basic_record['eventID'])
        self.assertEqual(record.name, self.basic_record['eventName'])
        self.assertEqual(record.source, self.basic_record['eventSource'])
        self.assertEqual(record.source_arn, self.basic_record['eventSourceARN'])
        self.assertEqual(record.region, self.basic_record['awsRegion'])
        self.assertEqual(record.version, self.basic_record['eventVersion'])
        self.assertEqual(record.invoke_identity_arn, self.basic_record['invokeIdentityArn'])
        self.assertEqual(record.schema_version, self.basic_record['kinesis']['kinesisSchemaVersion'])
        self.assertEqual(record.partition_key, self.basic_record['kinesis']['partitionKey'])
        self.assertEqual(record.time_stamp, self.basic_record['kinesis']['approximateArrivalTimestamp'])
        self.assertEqual(record.sequence_number, self.basic_record['kinesis']['sequenceNumber'])
        self.assertEqual(record.data, json.loads(base64.b64decode(self.basic_record['kinesis']['data']).decode('utf-8')))
        self.assertEqual(record.body, json.loads(base64.b64decode(self.basic_record['kinesis']['data']).decode('utf-8')))

    def test_record_prints(self):
        try:
            record = Record(self.basic_record)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import unittest

from acai_aws.apigateway.exception import ApiTimeOutException
from acai_aws.apigateway.request import Request
from acai_aws.apigateway.response import Response

from tests.mocks.apigateway import mock_request
from tests.mocks.apigateway.requirements import basic


class ApigatewayRequirementsTest(unittest.TestCase):
    basic_request = mock_request.get_basic()
    expected_data_class_result = {
        'hasErrors': False,
        'response': {
            'headers': {
                'Access-Control-Allow-Origin': '*',
                'Access-Control-Allow-Headers': '*'
            },
            'statusCode': 200,
            'isBase64Encoded': False,
            'body': {
                'requirements_basic': True
            }
        }
    }

    def test_requirements_decorator_has_attribute(self):
        self.assertTrue(hasattr(basic.post, 'requirements'))

    def test_requirements_runs_before(self):
        request = Request(self.basic_request)
        response = Response()
        basic.post(request, response)
        self.assertTrue(basic.before_call.has_been_called)

    def test_requirements_runs_after(self):
        request = Request(self.basic_request)
        response = Response()
        basic.post(request, response)
        self.assertTrue(basic.after_call.has_been_called)

    def test_requirements_runs_in_correct_order(self):
        request = Request(self.basic_request)
        response = Response()
        basic.post(request, response)
        self.assertEqual('before', basic.call_order[0])
        self.assertEqual('after', basic.call_order[1])

    def test_requirements_passes_after_data_class(self):
        request = Request(self.basic_request)
        response = Response()
        result = basic.post(request, response)
        self.assertEqual(str(self.expected_data_class_result), str(result))
    
    def test_requirements_global_timeout_raises_exception(self):
        request = Request(self.basic_request, None, 1)
        response = Response()
        try:
            basic.get(request, response)
            self.assertTrue(False)
        except ApiTimeOutException as error:
            self.assertTrue(isinstance(error, ApiTimeOutException))
        
    def test_requirements_local_timeout_raises_exception(self):
        event = mock_request.get_dynamic_event(method='patch')
        request = Request(event)
        response = Response()
        try:
            basic.patch(request, response)
            self.assertTrue(False)
        except ApiTimeOutException as error:
            self.assertTrue(isinstance(error, ApiTimeOutException))
        
    def test_requirements_local_overwrites_global_timeout_setting(self):
        event = mock_request.get_dynamic_event(method='patch')
        request = Request(event, None, 10)
        response = Response()
        try:
            basic.patch(request, response)
            self.assertTrue(False)
        except ApiTimeOutException as error:
            self.assertTrue(isinstance(error, ApiTimeOutException))    



from acai_aws.base.record import BaseRecord


class Record(BaseRecord):

    @property
    def name(self):
        return self._record.get('eventName')

    @property
    def source(self):
        return self._record.get('eventSource')

    @property
    def version(self):
        return self._record.get('eventVersion')

    @property
    def time(self):
        return self._record.get('eventTime')

    @property
    def region(self):
        return self._record.get('awsRegion')

    @property
    def request(self):
        return self._record.get('requestParameters')

    @property
    def response(self):
        return self._record.get('responseElements')

    @property
    def configuration_id(self):
        return self._record['s3'].get('configurationId')

    @property
    def object(self):
        return self._record['s3'].get('object')

    @property
    def bucket(self):
        return self._record['s3'].get('bucket', {}).get('name')

    @property
    def bucket_arn(self):
        return self._record['s3'].get('bucket', {}).get('arn')

    @property
    def bucket_owner(self):
        return self._record['s3'].get('bucket', {}).get('ownerIdentity', {}).get('principalId')

    @property
    def key(self):
        return self._record['s3'].get('object', {}).get('key')

    @property
    def schema_version(self):
        return self._record['s3'].get('s3SchemaVersion')

    @property
    def user_identity(self):
        return self._record['userIdentity'].get('principalId')

    @property
    def operation(self):
        if 'ObjectCreated' in self.name:
            return self.CREATED
        if 'ObjectRemoved' in self.name:
            return self.DELETED
        return self.UNKNOWN

    @property
    def body(self):
        return self._body

    @body.setter
    def body(self, body):
        self._body = body

    def __str__(self):
        return str({
            'name': self.name,
            'source': self.source,
            'version': self.version,
            'time': self.time,
            'region': self.region,
            'request': self.request,
            'response': self.response,
            'configuration_id': self.configuration_id,
            'object': self.object,
            'bucket': self.bucket,
            'schema_version': self.schema_version
        })



import unittest

from acai_aws.apigateway.openapi.input.validator import InputValidator


class MockInputArguments:

    def __init__(self):
        self.base = 'acai_aws/example'
        self.handlers = 'tests/mocks/apigateway/openapi/**/*.py'
        self.output = 'tests/mocks'
        self.format = 'json,yml'


class InputValidatorTest(unittest.TestCase):

    def setUp(self):
        self.validator = InputValidator()

    def test_validate_arguments_all_pass(self):
        try:
            inputs = MockInputArguments()
            self.validator.validate_arguments(inputs)
            self.assertTrue(True)
        except:
            self.assertTrue(False)

    def test_validate_arguments_glob_fails(self):
        try:
            inputs = MockInputArguments()
            inputs.handlers = 'tests/mocks/apigateway/openapi'
            self.validator.validate_arguments(inputs)
            self.assertTrue(False)
        except Exception as error:
            self.assertTrue(True)
            self.assertTrue('needs to be a glob pattern containing a "*.py"' in repr(error))

    def test_validate_arguments_directory_fails(self):
        try:
            inputs = MockInputArguments()
            inputs.output = 'tests/fail'
            self.validator.validate_arguments(inputs)
            self.assertTrue(False)
        except Exception as error:
            self.assertTrue(True)
            self.assertTrue('is not a valid directory path' in repr(error))



import unittest

from acai_aws.dynamodb.record import Record

from tests.mocks.dynamodb import mock_event


class DynamoDBRecordTest(unittest.TestCase):
    created_record = mock_event.get_created_event()['Records'][0]
    updated_record = mock_event.get_updated_event()['Records'][0]
    deleted_record = mock_event.get_deleted_event()['Records'][0]

    def test_record_accepts_event(self):
        record = Record(self.created_record)
        expected_keys = {'example_id': '123456789'}
        expected_body = {
            'example_id': '123456789',
            'note': 'Hosrawguw verrig zogupap ce so fajdis vub mos sif mawpowpug kif kihane.',
            'active': True,
            'personal': {
                'gender': 'male',
                'last_name': 'Mcneil',
                'first_name': 'Mannix'
            },
            'transportation': ['public-transit', 'car-access']
        }
        self.assertDictEqual(record.body, expected_body)
        self.assertDictEqual(record.new_image, expected_body)
        self.assertDictEqual(record.old_image, {})
        self.assertEqual(record.name, self.created_record['eventName'])
        self.assertEqual(record.source_arn, self.created_record['eventSourceARN'])
        self.assertEqual(record.source, self.created_record['eventSource'])
        self.assertEqual(record.version, self.created_record['eventVersion'])
        self.assertEqual(record.id, self.created_record['eventID'])
        self.assertEqual(record.region, self.created_record['awsRegion'])
        self.assertEqual(record.stream_view_type, self.created_record['dynamodb']['StreamViewType'])
        self.assertEqual(record.sequence_number, self.created_record['dynamodb']['SequenceNumber'])
        self.assertEqual(record.size_bytes, self.created_record['dynamodb']['SizeBytes'])
        self.assertEqual(record.keys, expected_keys)
        self.assertEqual(record.approximate_creation_time, self.created_record['dynamodb']['ApproximateCreationDateTime'])
        self.assertEqual(record.operation, record.CREATED)

    def test_record_accepts_updated_event(self):
        record = Record(self.updated_record)
        self.assertEqual(record.operation, record.UPDATED)

    def test_record_accepts_deleted_event(self):
        record = Record(self.deleted_record)
        self.assertEqual(record.operation, record.DELETED)

    def test_record_accepts_unknown_event(self):
        unknown_event = self.deleted_record.copy()
        del unknown_event['dynamodb']['OldImage']
        record = Record(unknown_event)
        self.assertEqual(record.operation, record.UNKNOWN)

    def test_record_prints(self):
        try:
            record = Record(self.created_record)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import os
from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='acai_aws',
    version=os.getenv('CIRCLE_TAG'),
    url='https://github.com/syngenta/acai-python.git',
    author='Paul Cruse III',
    author_email='paulcruse3@gmail.com',
    description='DRY, configurable, opinionated, minimalist framework for use with AWS Serverless Lambdas',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    python_requires='>=3.0',
    install_requires=[
        'boto3',
        'dynamodb_json',
        'jsonpickle',
        'jsonref',
        'jsonschema',
        'icecream',
        'pydantic',
        'pyyaml',
        'simplejson',
        'xmltodict'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11'
    ]
)



import json
import unittest
import urllib

import xmltodict

from acai_aws.apigateway.request import Request
from tests.mocks.apigateway import mock_request


class RequestTest(unittest.TestCase):
    websocket_aws_example = mock_request.get_aws_websocket_example()
    aws_example = mock_request.get_aws_example()
    basic_request = mock_request.get_basic()
    basic_offline = mock_request.get_basic_offline()
    basic_form = mock_request.get_basic_form()
    basic_xml = mock_request.get_basic_xml()
    basic_raw = mock_request.get_basic_raw()
    basic_graphql = mock_request.get_basic_graphql()
    basic_graphql_variables = mock_request.basic_graphql_variables()
    missing_inputs = mock_request.get_request_with_missing_fields()

    def test_method(self):
        request = Request(self.basic_request)
        self.assertEqual(request.method, self.basic_request['httpMethod'].lower())

    def test_cookies(self):
        request = Request(self.aws_example)
        self.assertEqual(request.cookies, self.aws_example['headers']['cookie'])

    def test_websocket_cookies(self):
        request = Request(self.websocket_aws_example)
        self.assertEqual(request.cookies, ';'.join(self.websocket_aws_example.get('cookies', [])))

    def test_content_type(self):
        request = Request(self.basic_request)
        self.assertEqual(request.content_type, self.basic_request['headers']['content-type'])

    def test_protocol(self):
        request = Request(self.aws_example)
        self.assertEqual(request.protocol, 'http')

    def test_websocket_protocol(self):
        request = Request(self.websocket_aws_example)
        self.assertEqual(request.protocol, 'http')

    def test_host_url(self):
        request = Request(self.aws_example)
        self.assertEqual(request.host_url, 'http://70ixmpl4fl.execute-api.us-east-2.amazonaws.com')

    def test_domain(self):
        request = Request(self.aws_example)
        self.assertEqual(request.domain, '70ixmpl4fl.execute-api.us-east-2.amazonaws.com')

    def test_stage(self):
        request = Request(self.aws_example)
        self.assertEqual(request.stage, 'Prod')

    def test_resource(self):
        request = Request(self.basic_request)
        self.assertEqual(request.resource, self.basic_request['resource'])

    def test_path(self):
        request = Request(self.basic_request)
        self.assertEqual(request.path, self.basic_request['path'])

    def test_route_default(self):
        request = Request(self.basic_request)
        self.assertEqual(request.route, f'/{self.basic_request["path"]}')

    def test_route_mutated(self):
        mutated = '/unit-test/v1/mutated'
        request = Request(self.basic_request)
        request.route = mutated
        self.assertEqual(request.route, mutated)

    def test_authorizer(self):
        request = Request(self.basic_request)
        self.assertDictEqual(request.authorizer, self.basic_request['requestContext']['authorizer'])

    def test_authorizer_offline(self):
        request = Request(self.basic_offline)
        self.assertDictEqual(request.authorizer, self.basic_offline['headers'])

    def test_headers(self):
        request = Request(self.basic_request)
        self.assertDictEqual(request.headers, self.basic_request['headers'])

    def test_body_json(self):
        request = Request(self.basic_request)
        self.assertDictEqual(request.body, json.loads(self.basic_request['body']))

    def test_json(self):
        request = Request(self.basic_request)
        self.assertDictEqual(request.json, json.loads(self.basic_request['body']))

    def test_body_form(self):
        request = Request(self.basic_form)
        self.assertDictEqual(request.body, dict(urllib.parse.parse_qsl(self.basic_form['body'])))

    def test_form(self):
        request = Request(self.basic_form)
        self.assertDictEqual(request.form, dict(urllib.parse.parse_qsl(self.basic_form['body'])))

    def test_body_xml(self):
        request = Request(self.basic_xml)
        self.assertDictEqual(request.body, xmltodict.parse(self.basic_xml['body']))

    def test_xml(self):
        request = Request(self.basic_xml)
        self.assertDictEqual(request.xml, xmltodict.parse(self.basic_xml['body']))

    def test_body_raw(self):
        request = Request(self.basic_raw)
        self.assertEqual(request.body, self.basic_raw['body'])

    def test_raw(self):
        request = Request(self.basic_raw)
        self.assertEqual(request.raw, self.basic_raw['body'])

    def test_body_graphql(self):
        request = Request(self.basic_graphql)
        self.assertEqual(request.body, self.basic_graphql['body'])

    def test_graphql(self):
        request = Request(self.basic_graphql)
        self.assertEqual(request.graphql, self.basic_graphql['body'])

    def test_graphql_base64(self):
        base64_graphql = self.basic_graphql.copy()
        base64_graphql['body'] = 'e3BsYXllcnN7bmFtZX19'  # used online tool to generate this
        request = Request(base64_graphql)
        self.assertEqual(request.graphql, self.basic_graphql['body'])

    def test_graphql_bad_base64(self):
        bad_base64_graphql = self.basic_graphql.copy()
        bad_base64_graphql['body'] = 1234
        request = Request(bad_base64_graphql)
        self.assertEqual(request.graphql, bad_base64_graphql['body'])

    def test_graphql_with_variables(self):
        request = Request(self.basic_graphql_variables)
        self.assertDictEqual(request.graphql, json.loads(self.basic_graphql_variables['body']))

    def test_query_params(self):
        request = Request(self.basic_request)
        self.assertDictEqual(request.query_params, self.basic_request['queryStringParameters'])

    def test_path_params(self):
        request = Request(self.basic_request)
        self.assertDictEqual(request.path_params, self.basic_request['pathParameters'])

    def test_path_params_setter(self):
        request = Request(self.basic_request)
        request.clear_path_params()
        request.path_params = ('hello', 'world')
        self.assertDictEqual(request.path_params, {'hello': 'world'})

    def test_path_params_clear(self):
        request = Request(self.basic_request)
        request.clear_path_params()
        self.assertDictEqual(request.path_params, {})

    def test_params(self):
        request = Request(self.basic_request)
        result = {'query': self.basic_request['queryStringParameters'], 'path': self.basic_request['pathParameters']}
        self.assertDictEqual(request.params, result)

    def test_context_default(self):
        request = Request(self.basic_request)
        self.assertDictEqual(request.context, {})

    def test_context_mutatable(self):
        context = {'hello': 'context'}
        request = Request(self.basic_request)
        request.context = context
        self.assertDictEqual(request.context, context)

    def test_event(self):
        request = Request(self.basic_request)
        self.assertDictEqual(
            request.event, {
                'headers': {
                    'x-api-key': 'SOME-KEY',
                    'content-type': 'application/json'
                },
                'requestContext': {
                    'resourceId': 't89kib',
                    'authorizer': {
                        'x-authorizer-key': 'SOME KEY',
                        'principalId': '9de3f415a97e410386dbef146e88744e',
                        'integrationLatency': 572
                    }
                },
                'path': 'unit-test/v1/basic',
                'pathParameters': {'proxy': 'hello'},
                'resource': '/{proxy+}',
                'httpMethod': 'GET',
                'queryStringParameters': {'name': 'me'},
                'body': '{"body_key": "body_value"}'
            }
        )

    def test_request(self):
        request = Request(self.basic_request)
        self.assertDictEqual(
            request.full, {
                'method': 'get',
                'resource': '/{proxy+}',
                'headers': {
                    'x-api-key': 'SOME-KEY',
                    'content-type': 'application/json'
                },
                'authorizer': {
                    'x-authorizer-key': 'SOME KEY',
                    'principalId': '9de3f415a97e410386dbef146e88744e',
                    'integrationLatency': 572
                },
                'params': {'query': {'name': 'me'}, 'path': {'proxy': 'hello'}},
                'body': {'body_key': 'body_value'},
                'context': {}
            }
        )

    def test_str(self):
        request = Request(self.basic_request)
        self.assertEqual(
            str(request),
            '{"method": "get", "resource": "/{proxy+}", "headers": {"x-api-key": "SOME-KEY", '
            '"content-type": "application/json"}, "authorizer": {"x-authorizer-key": "SOME KEY", '
            '"principalId": "9de3f415a97e410386dbef146e88744e", "integrationLatency": 572}, "params": {'
            '"query": {"name": "me"}, "path": {"proxy": "hello"}}, "body": {"body_key": "body_value"}, '
            '"context": {}}'
        )
    
    def test_defaults(self):
        request = Request(self.missing_inputs)
        self.assertEqual(request.method, '')
        self.assertEqual(request.resource, '')
        self.assertEqual(request.path, '')
        self.assertEqual(request.query_params, {})
    
    def test_timeout(self):
        request = Request(self.basic_request, None, 30)
        self.assertEqual(request.timeout, 30)
